###################################################
### chunk number 1: 
###################################################
options(width = 80)
library("sets")


###################################################
### chunk number 2: tuples
###################################################
## Do not quote strings
set_options("quote", FALSE)

## constructor
tuple(1,2,3, TRUE)
triple(1,2,3)
pair(Name = "David", Height = 185)
tuple_is_triple(triple(1,2,3))
tuple_is_ntuple(tuple(1,2,3,4), 4)

## converter
as.tuple(1:3)

## operations
c(tuple("a","b"), 1)
tuple(1,2,3) * tuple(2,3,4)
rep(tuple(1,2,3), 2)


###################################################
### chunk number 3: tuplesummary
###################################################
sum(tuple(1,2,3))
range(tuple(1,2,3))


###################################################
### chunk number 4: tupleouter
###################################################
tuple_outer(pair(1,2), triple(1,2,3))
tuple_outer(1:5, 1:4, "^")


###################################################
### chunk number 5: sets1
###################################################
## constructor
s <- set(1, 2, 3)
s

## named elements
snamed <- set(one = 1, 2, three = 3)
snamed

## named elements can directly be accessed
snamed[["one"]]

## a more complex set
set(c, "test", list(1, 2, 3))

## set of sets
set(set(), set(1))

## conversion functions
s2 <- as.set(2:5)
s2


###################################################
### chunk number 6: setpredicates
###################################################
set_is_empty(set())
set_is_equal(set(1), set(1))
set(1) == set(1)
set(1) != set(2)

set_is_subset(set(1), set(1,2))
set(1) <= set(1,2)
set(1,2) >= set(1)

set_is_proper_subset(set(1), set(1))
set(1) < set(1)
set(1,2) > set(1)

set_contains_element(set(1,2,3), 1)
1 %e% set(1,2,3)

set_contains_element(set(1,2,3), 1:4)
1:4 %e% set(1,2,3)


###################################################
### chunk number 7: setops
###################################################
length(s)
length(set())

## complement, union, intersection, symmetric difference:
s - 1
s + set("a")
s | set("a")
s & s2
s %D% s2
set(1,2,3) - set(1,2)

set_intersection(set(1,2,3), set(2,3,4), set(3,4,5))
set_union(set(1,2,3), set(2,3,4), set(3,4,5))
set_symdiff(set(1,2,3), set(2,3,4), set(3,4,5))

## Cartesian product
s * s2
s * s
s ^ 2 # same as above
s ^ 3

## power set
2 ^ s

## subsets:
set_combn(as.set(1:3),2)

## closure and reduction (under union):
cl <- closure(set(set(1), set(2), set(3)))
print(cl)
reduction(cl)


###################################################
### chunk number 8: setsummary
###################################################
sum(s)
range(s)


###################################################
### chunk number 9: setouter
###################################################
set_outer(set(1,2), set(1,2,3), "/")
X <- set_outer(set(1,2), set(1,2,3), set)
X[[2,3]]
set_outer(2 ^ set(1,2,3), set_is_subset)


###################################################
### chunk number 10: iterators
###################################################
sapply(s, sqrt)
for (i in s) print(i)


###################################################
### chunk number 11: 
###################################################
X <- c("A", "B", "C")

## ordinary set (X is converted to a set internally).
gset(support = X)

## multiset
multi <- 1:3
gset(support = X, memberships = multi)

## fuzzy set
ms <- c(0.1, 0.3, 1)
gset(support = X, memberships = ms)

## fuzzy set using a membership function
f <- function(x) switch(x, A = 0.1, B = 0.2, C = 1, 0)
gset(support = X, charfun = f)

## fuzzy multiset
## Here, the membership argument expects a list of membership grades,
## either specified as vectors, or as multisets.
ms2 <- list(c(0.1, 0.3, 0.4), c(1, 1),
            gset(support = ms, memberships = multi))
gset(support = X, memberships = ms2)


###################################################
### chunk number 12: 
###################################################
X <- gset(c("A", "B", "C"), 4:6)
print(X)
Y <- gset(c("B", "C", "D"), 1:3)
print(Y)

## union vs. sum
gset_union(X, Y)
gset_sum(X, Y)

## intersection vs. difference
gset_intersection(X, Y)
gset_difference(X, Y)

## sum and difference for fuzzy sets
X <- gset("a", 0.3)
Y <- gset(c("a", "b"), c(0.3, 0.4))
gset_sum(X, Y)
gset_sum(X, Y, set("a"))
gset_difference(Y, X)


###################################################
### chunk number 13: 
###################################################
X - Y
gset_complement(X, Y)


###################################################
### chunk number 14: 
###################################################
X <- gset(c("A", "B", "C"), c(0.3, 0.5, 0.8))
print(X)
Y <- gset(c("B", "C", "D"), c(0.1, 0.3, 0.9))
print(Y)

## Zadeh-logic (default)
gset_intersection(X, Y)
gset_union(X, Y)
gset_complement(X, Y)
!X

## switch logic
fuzzy_logic("Fodor")

## Fodor-logic
gset_intersection(X, Y)
gset_union(X, Y)
gset_complement(X, Y)
!X


###################################################
### chunk number 15: 
###################################################
cut(X, 0.5)
## default level is 1, so this filters all "crisp" elements:
cut(X)


###################################################
### chunk number 16: fuzzyplot
###################################################
plot(X)


###################################################
### chunk number 17: 
###################################################
plot(X)


###################################################
### chunk number 18: 
###################################################
## restore string quoting
set_options("quote", TRUE)

## default behavior of sets: matching of elements is very strict
## Note that on most systems, 3.3 - 2.2 != 1.1
x <- set("1", 1L, 1, 3.3 - 2.2, 1.1)
print(x)

y <- set(1, 1.1, 2L, "2")
print(y)
1L %e% y

set_union(x, y)
set_intersection(x, y)
set_complement(x, y)

### Now use the more sloppy match()-function (i.e., `==`)
### Note that 1 == "1" == 1L ...
X <- cset(x, matchfun = match)
print(X)
Y <- cset(y, matchfun = match)
print(Y)
1L %e% Y

cset_union(X, Y)
cset_intersection(X, Y)
cset_complement(X, Y)

## Same using all.equal().
## This is a non-vectorized predicate, so use make_matchfun
## to generate a vectorized version:
FUN <- make_matchfun(function(x, y) isTRUE(all.equal(x, y)))
X <- cset(x, matchfun = FUN)
print(X)
Y <- cset(y, matchfun = FUN)
print(Y)
1L %e% Y

cset_union(X, Y)
cset_intersection(X, Y)
cset_complement(X, Y)


###################################################
### chunk number 19: 
###################################################
### change default functions via set_option
set_options("matchfun", match)
cset(x)
cset(y)

cset(1:3) <= cset(c(1,2,3))

### restore package defaults
set_options("matchfun", NULL)


###################################################
### chunk number 20: 
###################################################
## simple example using a permutation index vector
cset(letters[1:5], orderfun = 5:1)

### customized order function
FUN <- function(x) order(as.character(x), decreasing = TRUE)
Z <- cset(letters[1:5], orderfun = FUN)
print(Z)
as.character(Z)


###################################################
### chunk number 21: 
###################################################
o <- ordered(c("a", "b", "a"), levels = c("b", "a"))
as.set(o)
as.cset(o)


###################################################
### chunk number 22: 
###################################################
as.cset(c("A", "quick", "brown", "fox"))
as.cset(c("A", "quick", "brown", "fox", "quick"))


